import re

from abc import ABC, abstractmethod
import sys 
sys.path.append(r'C:\Users\HP\Downloads\ReportingService\ReportingService\Frontend\Neuer Ordner\Neuer Ordner\modern-dev-tdd-assignments')
from Project_Test.stringCalculator import StringCalculator as DaniCalc

class IStringCalculator(ABC):
    @abstractmethod
    def add(self, numbers: str) -> int:
        pass

class StringCalculator(IStringCalculator):
    def add(self, numbers: str) -> int:
        if not numbers:
            return 0

        if numbers.startswith("//"):
            if "\n" not in numbers:
                raise ValueError("Ungültiges Format: Nicht vollständig")
            delimiter_end_index = numbers.index("\n")
            delimiter_part = numbers[2:delimiter_end_index]
            if delimiter_part.startswith("[") and delimiter_part.endswith("]"):
                delimiter = delimiter_part[1:-1]
            else:
                delimiter = delimiter_part
            numbers = numbers[delimiter_end_index + 1:]
            numbers = numbers.replace(delimiter, ",")

        numbers = numbers.replace("\n", ",")

        # Split the string by commas, convert each value to an integer, and sum them up
        try:
            numbers_list = list(map(int, numbers.split(",")))
        except ValueError:
            raise ValueError("Ungültiges Zahlenformat: Enthält nicht-numerische Werte")

        negative_numbers = [num for num in numbers_list if num < 0]
        if negative_numbers:
            raise ValueError(f"Negative nicht erlaubt: {negative_numbers}")

        numbers_list = [num for num in numbers_list if num <= 1000]

        return sum(numbers_list)
    

import unittest

class TestStringCalculator(unittest.TestCase):
    def setUp(self):
        self.calculator = DaniCalc()

    def test_add_empty_string(self):
        self.assertEqual(self.calculator.add(""), 0)

    def test_add_single_number(self):
        self.assertEqual(self.calculator.add("1"), 1)

    def test_add_two_numbers(self):
        self.assertEqual(self.calculator.add("10,20"), 30)

    def test_add_multiple_numbers(self):
        self.assertEqual(self.calculator.add("1,2,3"), 6)
        self.assertEqual(self.calculator.add("10,20,30,40"), 100)

    def test_add_with_newline_separator(self):
        self.assertEqual(self.calculator.add("1\n2,3"), 6)
        self.assertEqual(self.calculator.add("10\n20\n30"), 60)

    def test_add_single_negative_number(self):
        with self.assertRaises(ValueError) as context:
            self.calculator.add("1,-2,3")
        print(str(context.exception))
        self.assertEqual(str(context.exception), "Negative nicht erlaubt: [-2]")

    def test_add_multiple_negative_numbers(self):
        with self.assertRaises(ValueError) as context:
            self.calculator.add("-10\n-20,-30")
        print(str(context.exception))
        self.assertEqual(str(context.exception), "Negative nicht erlaubt: [-10, -20, -30]")

    def test_add_with_custom_delimiter(self):
        self.assertEqual(self.calculator.add("//;\n1;2"), 3)
        self.assertEqual(self.calculator.add("//x\n7x8\n9"), 24)

    def test_invalid_custom_delimiter_format(self):
        with self.assertRaises(ValueError) as context:
            self.calculator.add("//;1;2")
        print(str(context.exception))

    def test_ignore_numbers_greater_than_1000(self):
        self.assertEqual(self.calculator.add("2,1001"), 2)
        self.assertEqual(self.calculator.add("1002,50200"), 0)

    def test_add_with_custom_delimiter_multiple_characters(self):
        self.assertEqual(self.calculator.add("//[**]\n1**2**3"), 6)
        self.assertEqual(self.calculator.add("//[###]\n10###20###30"), 60)

if __name__ == "__main__":
    unittest.main()

# python -m unittest tests.test_stringcalculator
# python -m unittest discover -s tests -v